@@ -1,5 +1,5 @@
 /* gif.h - Interface to the LCDF GIF library.
-   Copyright (C) 1997-2011 Eddie Kohler, kohler@cs.ucla.edu
+   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of the LCDF GIF library.
 
    The GIF library is free software. It is distributed under the GNU General
@@ -58,6 +58,7 @@ struct Gif_Stream {
     unsigned errors;
 
     int userflags;
+    const char* landmark;
     int refcount;
 
 };
@@ -125,6 +126,7 @@ void		Gif_DeleteImage(Gif_Image *gfi);
 int		Gif_AddImage(Gif_Stream *gfs, Gif_Image *gfi);
 void		Gif_RemoveImage(Gif_Stream *gfs, int i);
 Gif_Image *	Gif_CopyImage(Gif_Image *gfi);
+void            Gif_MakeImageEmpty(Gif_Image* gfi);
 
 Gif_Image *	Gif_GetImage(Gif_Stream *gfs, int i);
 Gif_Image *	Gif_GetNamedImage(Gif_Stream *gfs, const char *name);
@@ -135,19 +137,21 @@ int		Gif_ImageNumber(Gif_Stream *gfs, Gif_Image *gfi);
 #define		Gif_ImageDelay(gfi)		((gfi)->delay)
 #define		Gif_ImageUserData(gfi)		((gfi)->userdata)
 #define		Gif_SetImageUserData(gfi, v)	((gfi)->userdata = v)
+int             Gif_ImageColorBound(const Gif_Image* gfi);
 
-typedef		void (*Gif_ReadErrorHandler)(int is_error,
-					     const char *error_text,
-					     int frame_number,
-					     void *user_data);
+typedef		void (*Gif_ReadErrorHandler)(Gif_Stream* gfs,
+                                             Gif_Image* gfi,
+					     int is_error,
+					     const char* error_text);
 
 typedef struct {
     int flags;
     void *padding[7];
 } Gif_CompressInfo;
 
-#define		Gif_UncompressImage(gfi)     Gif_FullUncompressImage((gfi),0,0)
-int		Gif_FullUncompressImage(Gif_Image *gfs,Gif_ReadErrorHandler,void*);
+#define		Gif_UncompressImage(gfs, gfi) Gif_FullUncompressImage((gfs),(gfi),0)
+int		Gif_FullUncompressImage(Gif_Stream* gfs, Gif_Image* gfi,
+                                        Gif_ReadErrorHandler handler);
 int		Gif_CompressImage(Gif_Stream *gfs, Gif_Image *gfi);
 int		Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi,
 				      const Gif_CompressInfo *gcinfo);
@@ -155,7 +159,7 @@ void		Gif_ReleaseUncompressedImage(Gif_Image *gfi);
 void		Gif_ReleaseCompressedImage(Gif_Image *gfi);
 int		Gif_SetUncompressedImage(Gif_Image *gfi, uint8_t *data,
 			void (*free_data)(void *), int data_interlaced);
-int		Gif_CreateUncompressedImage(Gif_Image *gfi);
+int		Gif_CreateUncompressedImage(Gif_Image* gfi, int data_interlaced);
 
 int		Gif_ClipImage(Gif_Image *gfi, int l, int t, int w, int h);
 
@@ -203,6 +207,8 @@ int		Gif_ColorEq(Gif_Color *, Gif_Color *);
 #define		GIF_COLOREQ(c1, c2) \
 ((c1)->gfc_red==(c2)->gfc_red && (c1)->gfc_green==(c2)->gfc_green && \
  (c1)->gfc_blue==(c2)->gfc_blue)
+#define         GIF_SETCOLOR(c, r, g, b) \
+((c)->gfc_red = (r), (c)->gfc_green = (g), (c)->gfc_blue = (b))
 
 int		Gif_FindColor(Gif_Colormap *, Gif_Color *);
 int		Gif_AddColor(Gif_Colormap *, Gif_Color *, int look_from);
@@ -226,24 +232,26 @@ int		Gif_AddComment(Gif_Comment *, const char *, int);
 /** GIF_EXTENSION **/
 
 struct Gif_Extension {
-
     int kind;			/* negative kinds are reserved */
-    char *application;
-    uint8_t *data;
+    char* appname;
+    int applength;
+    uint8_t* data;
     uint32_t length;
     int position;
+    int packetized;
 
     Gif_Stream *stream;
     Gif_Extension *next;
     void (*free_data)(void *);
-
 };
 
 
-Gif_Extension *	Gif_NewExtension(int, const char *);
-void		Gif_DeleteExtension(Gif_Extension *);
-int		Gif_AddExtension(Gif_Stream *, Gif_Extension *, int);
-Gif_Extension * Gif_GetExtension(Gif_Stream *, int, Gif_Extension *);
+Gif_Extension*  Gif_NewExtension(int kind, const char* appname, int applength);
+void		Gif_DeleteExtension(Gif_Extension* gfex);
+int		Gif_AddExtension(Gif_Stream* gfs, Gif_Extension* gfex,
+                                 int imageno);
+Gif_Extension*  Gif_GetExtension(Gif_Stream* gfs, int kind,
+                                 Gif_Extension* position);
 
 
 /** READING AND WRITING **/
@@ -262,12 +270,14 @@ struct Gif_Record {
 #define GIF_WRITE_OPTIMIZE		4
 #define GIF_WRITE_SHRINK		8
 
-Gif_Stream *	Gif_ReadFile(FILE *);
-Gif_Stream *	Gif_FullReadFile(FILE *, int flags, Gif_ReadErrorHandler,
-				 void *);
-Gif_Stream *	Gif_ReadRecord(const Gif_Record *);
-Gif_Stream *	Gif_FullReadRecord(const Gif_Record *, int flags,
-				   Gif_ReadErrorHandler, void *);
+void            Gif_SetErrorHandler(Gif_ReadErrorHandler handler);
+Gif_Stream*	Gif_ReadFile(FILE* f);
+Gif_Stream*	Gif_FullReadFile(FILE* f, int flags, const char* landmark,
+                                 Gif_ReadErrorHandler handler);
+Gif_Stream*	Gif_ReadRecord(const Gif_Record* record);
+Gif_Stream*	Gif_FullReadRecord(const Gif_Record* record, int flags,
+				   const char* landmark,
+                                   Gif_ReadErrorHandler handler);
 int		Gif_WriteFile(Gif_Stream *gfs, FILE *f);
 int		Gif_FullWriteFile(Gif_Stream *gfs,
 				  const Gif_CompressInfo *gcinfo, FILE *f);
@@ -297,23 +307,19 @@ void		Gif_Debug(char *x, ...);
 #define		GIF_DEBUG(x)
 #endif
 
+void*           Gif_Realloc(void* p, size_t s, size_t n,
+                            const char* file, int line);
+void            Gif_Free(void* p);
+#if !GIF_ALLOCATOR_DEFINED
+# define        Gif_Free        free
+#endif
+
 #ifndef Gif_New
-# ifndef xmalloc
-#  define xmalloc		malloc
-#  define xrealloc		realloc
-#  define xfree			free
-# endif
-# define Gif_New(t)		((t *)xmalloc(sizeof(t)))
-# define Gif_NewArray(t, n)	((t *)xmalloc(sizeof(t) * (n)))
-# define Gif_ReArray(p, t, n)	((p)=((t*)xrealloc((void*)(p),sizeof(t)*(n))))
-#endif
-#ifndef Gif_DeleteFunc
-# define Gif_DeleteFunc		(&xfree)
-# define Gif_DeleteArrayFunc	(&xfree)
-#endif
-#ifndef Gif_Delete
-# define Gif_Delete(p)		(*Gif_DeleteFunc)((void *)(p))
-# define Gif_DeleteArray(p)	(*Gif_DeleteArrayFunc)((void *)(p))
+# define Gif_New(t)		((t*) Gif_Realloc(0, sizeof(t), 1, __FILE__, __LINE__))
+# define Gif_NewArray(t, n)	((t*) Gif_Realloc(0, sizeof(t), (n), __FILE__, __LINE__))
+# define Gif_ReArray(p, t, n)	((p)=(t*) Gif_Realloc((void*) (p), sizeof(t), (n), __FILE__, __LINE__))
+# define Gif_Delete(p)          Gif_Free((void*) (p))
+# define Gif_DeleteArray(p)     Gif_Free((void*) (p))
 #endif
 
 #ifdef __cplusplus
