@@ -1,5 +1,5 @@
 /* gifdiff.c - Gifdiff compares GIF images for identical appearance.
-   Copyright (C) 1998-2013 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1998-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of gifdiff, in the gifsicle package.
 
    Gifdiff is free software. It is distributed under the GNU Public License,
@@ -34,12 +34,12 @@ const Clp_Option options[] = {
   { "version", 'v', VERSION_OPT, 0, 0 }
 };
 
-static const char *program_name;
+const char *program_name;
 
 static const char *filename1;
 static const char *filename2;
 
-static int screen_width, screen_height;
+static unsigned screen_width, screen_height;
 #define TRANSP (0)
 
 static uint16_t *gdata[2];
@@ -50,6 +50,8 @@ static uint16_t *line;
 static int brief;
 static int ignore_redundancy;
 
+static Clp_Parser* clp;
+
 
 static void
 combine_colormaps(Gif_Colormap *gfcm, Gif_Colormap *newcm)
@@ -130,7 +132,7 @@ apply_image(int is_second, Gif_Stream *gfs, int imageno, uint16_t background)
   }
 
   /* uncompress and clip */
-  Gif_UncompressImage(gfi);
+  Gif_UncompressImage(gfs, gfi);
   Gif_ClipImage(gfi, 0, 0, screen_width, screen_height);
 
   any_change = imageno == 0;
@@ -255,6 +257,13 @@ compare(Gif_Stream *s1, Gif_Stream *s2)
     return DIFFERENT;
   }
 
+  if (s1->screen_width == 0 || s1->screen_height == 0
+      || s2->screen_width == 0 || s2->screen_height == 0) {
+    /* paranoia -- don't think this can happen */
+    different("zero screen sizes");
+    return DIFFERENT;
+  }
+
   /* Create arrays for the image data */
   screen_width = s1->screen_width;
   screen_height = s1->screen_height;
@@ -315,7 +324,7 @@ compare(Gif_Stream *s1, Gif_Stream *s2)
     /* compare pixels */
     if (memcmp(gdata[0], gdata[1],
 	       screen_width * screen_height * sizeof(uint16_t)) != 0) {
-      int d, c = screen_width * screen_height;
+      unsigned d, c = screen_width * screen_height;
       uint16_t *d1 = gdata[0], *d2 = gdata[1];
       for (d = 0; d < c; d++, d1++, d2++)
 	if (*d1 != *d2) {
@@ -368,27 +377,23 @@ compare(Gif_Stream *s1, Gif_Stream *s2)
 }
 
 
-void
-short_usage(void)
-{
-  fprintf(stderr, "Usage: %s [OPTION]... FILE1 FILE2\n\
-Try '%s --help' for more information.\n",
-	  program_name, program_name);
+void short_usage(void) {
+    Clp_fprintf(clp, stderr, "Usage: %s [OPTION]... FILE1 FILE2\n\
+Try %<%s --help%> for more information.\n",
+                program_name, program_name);
 }
 
-void
-usage(void)
-{
-  printf("\
-'Gifdiff' compares two GIF files (either images or animations) for identical\n\
+void usage(void) {
+    Clp_fprintf(clp, stdout, "\
+%<Gifdiff%> compares two GIF files (either images or animations) for identical\n\
 visual appearance. An animation and an optimized version of the same animation\n\
 should compare as the same. Gifdiff exits with status 0 if the images are\n\
-the same, 1 if they're different, and 2 if there was some error.\n\
+the same, 1 if they%,re different, and 2 if there was some error.\n\
 \n\
 Usage: %s [OPTION]... FILE1 FILE2\n\n", program_name);
-  printf("\
+    Clp_fprintf(clp, stdout, "\
 Options:\n\
-  -q, --brief                   Don't report detailed differences.\n\
+  -q, --brief                   Don%,t report detailed differences.\n\
   -w, --ignore-redundancy       Ignore differences in redundant frames.\n\
   -h, --help                    Print this message and exit.\n\
   -v, --version                 Print version number and exit.\n\
@@ -397,48 +402,42 @@ Report bugs to <ekohler@gmail.com>.\n");
 }
 
 
-void
-fatal_error(char *message, ...)
-{
-  va_list val;
-  va_start(val, message);
-  fprintf(stderr, "%s: ", program_name);
-  vfprintf(stderr, message, val);
-  fputc('\n', stderr);
-  exit(2);			/* exit(2) for trouble */
+void fatal_error(const char* format, ...) {
+    char buf[BUFSIZ];
+    int n = snprintf(buf, BUFSIZ, "%s: ", program_name);
+    va_list val;
+    va_start(val, format);
+    Clp_vsnprintf(clp, buf + n, BUFSIZ - n, format, val);
+    va_end(val);
+    fputs(buf, stderr);
+    exit(2);			/* exit(2) for trouble */
 }
 
-void
-error(char *message, ...)
-{
-  va_list val;
-  va_start(val, message);
-  fprintf(stderr, "%s: ", program_name);
-  vfprintf(stderr, message, val);
-  fputc('\n', stderr);
-}
-
-void
-warning(char *message, ...)
-{
-  va_list val;
-  va_start(val, message);
-  fprintf(stderr, "%s: warning: ", program_name);
-  vfprintf(stderr, message, val);
-  fputc('\n', stderr);
+void error(const char* format, ...) {
+    char buf[BUFSIZ];
+    int n = snprintf(buf, BUFSIZ, "%s: ", program_name);
+    va_list val;
+    va_start(val, format);
+    Clp_vsnprintf(clp, buf + n, BUFSIZ - n, format, val);
+    va_end(val);
+    fputs(buf, stderr);
 }
 
 static int gifread_error_count;
 
 static void
-gifread_error(int is_error, const char *message, int which_image, void *thunk)
+gifread_error(Gif_Stream* gfs, Gif_Image* gfi,
+              int is_error, const char *message)
 {
   static int last_is_error = 0;
   static int last_which_image = 0;
   static char last_message[256];
   static int different_error_count = 0;
   static int same_error_count = 0;
-  const char *filename = (const char *)thunk;
+  int which_image = Gif_ImageNumber(gfs, gfi);
+  const char *filename = gfs->landmark;
+  if (gfs && which_image < 0)
+    which_image = gfs->nimages;
 
   if (gifread_error_count == 0) {
     last_which_image = -1;
@@ -451,11 +450,11 @@ gifread_error(int is_error, const char *message, int which_image, void *thunk)
       && (last_which_image != which_image || message == 0
 	  || strcmp(message, last_message) != 0)) {
     const char *etype = last_is_error ? "error" : "warning";
-    error("While reading '%s' frame #%d:", filename, last_which_image);
+    error("While reading %<%s%> frame #%d:\n", filename, last_which_image);
     if (same_error_count == 1)
-      error("  %s: %s", etype, last_message);
+      error("  %s: %s\n", etype, last_message);
     else if (same_error_count > 0)
-      error("  %s: %s (%d times)", etype, last_message, same_error_count);
+      error("  %s: %s (%d times)\n", etype, last_message, same_error_count);
     same_error_count = 0;
     last_message[0] = 0;
   }
@@ -471,7 +470,7 @@ gifread_error(int is_error, const char *message, int which_image, void *thunk)
     last_message[0] = 0;
 
   if (different_error_count == 11 && message) {
-    error("(more errors while reading '%s')", filename);
+    error("(more errors while reading %<%s%>)\n", filename);
     different_error_count++;
   }
 }
@@ -488,7 +487,7 @@ read_stream(const char **filename)
 #ifndef OUTPUT_GIF_TO_TERMINAL
 	extern int isatty(int);
 	if (isatty(fileno(stdin))) {
-	    fatal_error("<stdin>: is a terminal");
+	    fatal_error("<stdin>: is a terminal\n");
 	    return NULL;
 	}
 #endif
@@ -505,13 +504,12 @@ read_stream(const char **filename)
     } else {
 	f = fopen(*filename, "rb");
 	if (!f)
-	    fatal_error("%s: %s", *filename, strerror(errno));
+	    fatal_error("%s: %s\n", *filename, strerror(errno));
     }
     gifread_error_count = 0;
-    gfs = Gif_FullReadFile(f, GIF_READ_COMPRESSED, gifread_error, (void *) *filename);
-    gifread_error(-1, 0, -1, (void *) *filename); /* print out last error message */
+    gfs = Gif_FullReadFile(f, GIF_READ_COMPRESSED, *filename, gifread_error);
     if (!gfs)
-	fatal_error("'%s' doesn't seem to contain a GIF", *filename);
+	fatal_error("%s: file not in GIF format\n", *filename);
     return gfs;
 }
 
@@ -523,8 +521,8 @@ main(int argc, char *argv[])
   const char **inputp;
   Gif_Stream *gfs1, *gfs2;
 
-  Clp_Parser *clp =
-    Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
+  clp = Clp_NewParser(argc, (const char * const *)argv,
+                      sizeof(options) / sizeof(options[0]), options);
 
   program_name = Clp_ProgramName(clp);
   brief = 0;
@@ -540,7 +538,7 @@ main(int argc, char *argv[])
 
      case VERSION_OPT:
       printf("gifdiff (LCDF Gifsicle) %s\n", VERSION);
-      printf("Copyright (C) 1998-2013 Eddie Kohler\n\
+      printf("Copyright (C) 1998-2014 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -557,7 +555,7 @@ particular purpose.\n");
 
      case Clp_NotOption:
       if (how_many_inputs == 2) {
-	error("too many file arguments");
+	error("too many file arguments\n");
 	goto bad_option;
       }
       inputp = (how_many_inputs == 0 ? &filename1 : &filename2);
@@ -583,9 +581,9 @@ particular purpose.\n");
  done:
 
   if (how_many_inputs < 2)
-    fatal_error("need exactly 2 file arguments");
+    fatal_error("need exactly 2 file arguments\n");
   if (filename1 == 0 && filename2 == 0)
-    fatal_error("can't read both files from stdin");
+    fatal_error("can%,t read both files from stdin\n");
 
   gfs1 = read_stream(&filename1);
   gfs2 = read_stream(&filename2);
@@ -596,8 +594,5 @@ particular purpose.\n");
 
   Gif_DeleteStream(gfs1);
   Gif_DeleteStream(gfs2);
-#ifdef DMALLOC
-  dmalloc_report();
-#endif
   return status;
 }
