@@ -1,5 +1,5 @@
 /* giffunc.c - General functions for the GIF library.
-   Copyright (C) 1997-2013 Eddie Kohler, kohler@cs.ucla.edu
+   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of the LCDF GIF library.
 
    The LCDF GIF library is free software. It is distributed under the GNU
@@ -35,6 +35,7 @@ Gif_NewStream(void)
   gfs->errors = 0;
   gfs->userflags = 0;
   gfs->refcount = 0;
+  gfs->landmark = 0;
   return gfs;
 }
 
@@ -55,7 +56,7 @@ Gif_NewImage(void)
   gfi->interlace = 0;
   gfi->img = 0;
   gfi->image_data = 0;
-  gfi->free_image_data = Gif_DeleteArrayFunc;
+  gfi->free_image_data = Gif_Free;
   gfi->compressed_len = 0;
   gfi->compressed = 0;
   gfi->free_compressed = 0;
@@ -116,23 +117,32 @@ Gif_NewComment(void)
 
 
 Gif_Extension *
-Gif_NewExtension(int kind, const char *app_name)
+Gif_NewExtension(int kind, const char* appname, int applength)
 {
-  Gif_Extension *gfex = Gif_New(Gif_Extension);
-  if (!gfex)
-    return 0;
-  gfex->kind = app_name ? 255 : kind;
-  gfex->application = Gif_CopyString(app_name);
-  gfex->data = 0;
-  gfex->position = 0;
-  gfex->stream = 0;
-  gfex->next = 0;
-  gfex->free_data = 0;
-  if (!gfex->application && app_name) {
-    Gif_DeleteExtension(gfex);
-    return 0;
-  }
-  return gfex;
+    Gif_Extension *gfex = Gif_New(Gif_Extension);
+    if (!gfex)
+        return 0;
+    gfex->kind = kind;
+    if (appname
+        && (gfex->appname = (char*) Gif_NewArray(char, applength + 1))) {
+        memcpy(gfex->appname, appname, applength);
+        gfex->appname[applength] = 0;
+        gfex->applength = applength;
+    } else {
+        gfex->appname = 0;
+        gfex->applength = 0;
+    }
+    gfex->data = 0;
+    gfex->position = 0;
+    gfex->stream = 0;
+    gfex->next = 0;
+    gfex->free_data = 0;
+    gfex->packetized = 0;
+    if (!gfex->appname && appname) {
+        Gif_DeleteExtension(gfex);
+        return 0;
+    }
+    return gfex;
 }
 
 
@@ -184,6 +194,16 @@ Gif_RemoveImage(Gif_Stream *gfs, int inum)
 }
 
 
+int
+Gif_ImageColorBound(const Gif_Image* gfi)
+{
+    if (gfi->compressed)
+        return 1 << gfi->compressed[0];
+    else
+        return 256;
+}
+
+
 int
 Gif_AddCommentTake(Gif_Comment *gfcom, char *x, int xlen)
 {
@@ -246,11 +266,12 @@ Gif_AddExtension(Gif_Stream *gfs, Gif_Extension *gfex, int pos)
 int
 Gif_ImageNumber(Gif_Stream *gfs, Gif_Image *gfi)
 {
-  int i;
-  for (i = 0; i < gfs->nimages; i++)
-    if (gfs->images[i] == gfi)
-      return i;
-  return -1;
+    int i;
+    if (gfs && gfi)
+        for (i = 0; i != gfs->nimages; ++i)
+            if (gfs->images[i] == gfi)
+                return i;
+    return -1;
 }
 
 
@@ -379,8 +400,8 @@ Gif_CopyImage(Gif_Image *src)
   dest->interlace = src->interlace;
   if (src->img) {
     dest->img = Gif_NewArray(uint8_t *, dest->height + 1);
-    dest->image_data = Gif_NewArray(uint8_t, dest->width * dest->height);
-    dest->free_image_data = Gif_DeleteArrayFunc;
+    dest->image_data = Gif_NewArray(uint8_t, (size_t) dest->width * (size_t) dest->height);
+    dest->free_image_data = Gif_Free;
     if (!dest->img || !dest->image_data)
       goto failure;
     for (i = 0, data = dest->image_data; i < dest->height; i++) {
@@ -395,7 +416,7 @@ Gif_CopyImage(Gif_Image *src)
       dest->compressed = src->compressed;
     else {
       dest->compressed = Gif_NewArray(uint8_t, src->compressed_len);
-      dest->free_compressed = Gif_DeleteArrayFunc;
+      dest->free_compressed = Gif_Free;
       memcpy(dest->compressed, src->compressed, src->compressed_len);
     }
     dest->compressed_len = src->compressed_len;
@@ -409,6 +430,16 @@ Gif_CopyImage(Gif_Image *src)
 }
 
 
+void Gif_MakeImageEmpty(Gif_Image* gfi) {
+    Gif_ReleaseUncompressedImage(gfi);
+    Gif_ReleaseCompressedImage(gfi);
+    gfi->width = gfi->height = 1;
+    gfi->transparent = 0;
+    Gif_CreateUncompressedImage(gfi, 0);
+    gfi->img[0][0] = 0;
+}
+
+
 /** DELETION **/
 
 typedef struct Gif_DeletionHook {
@@ -513,7 +544,7 @@ Gif_DeleteExtension(Gif_Extension *gfex)
     return;
   if (gfex->data && gfex->free_data)
     (*gfex->free_data)(gfex->data);
-  Gif_DeleteArray(gfex->application);
+  Gif_DeleteArray(gfex->appname);
   if (gfex->stream) {
     Gif_Stream *gfs = gfex->stream;
     Gif_Extension *prev, *trav;
@@ -730,9 +761,9 @@ Gif_SetUncompressedImage(Gif_Image *gfi, uint8_t *image_data,
 			 void (*free_data)(void *), int data_interlaced)
 {
   /* NB does not affect compressed image (and must not) */
-  int i;
-  int width = gfi->width;
-  int height = gfi->height;
+  unsigned i;
+  unsigned width = gfi->width;
+  unsigned height = gfi->height;
   uint8_t **img;
 
   Gif_ReleaseUncompressedImage(gfi);
@@ -758,11 +789,11 @@ Gif_SetUncompressedImage(Gif_Image *gfi, uint8_t *image_data,
 }
 
 int
-Gif_CreateUncompressedImage(Gif_Image *gfi)
+Gif_CreateUncompressedImage(Gif_Image *gfi, int data_interlaced)
 {
-  uint8_t *data = Gif_NewArray(uint8_t, gfi->width * gfi->height);
-  return Gif_SetUncompressedImage(gfi, data, Gif_DeleteArrayFunc,
-				  gfi->interlace);
+    size_t sz = (size_t) gfi->width * (size_t) gfi->height;
+    uint8_t *data = Gif_NewArray(uint8_t, sz ? sz : 1);
+    return Gif_SetUncompressedImage(gfi, data, Gif_Free, data_interlaced);
 }
 
 void
@@ -775,13 +806,31 @@ Gif_InitCompressInfo(Gif_CompressInfo *gcinfo)
 void
 Gif_Debug(char *x, ...)
 {
-  va_list val;
-  va_start(val, x);
-  vfprintf(stderr, x, val);
-  fputc(' ', stderr);
-  va_end(val);
+    va_list val;
+    va_start(val, x);
+    vfprintf(stderr, x, val);
+    fputc(' ', stderr);
+    va_end(val);
 }
 
+
+#if !GIF_ALLOCATOR_DEFINED
+void* Gif_Realloc(void* p, size_t s, size_t n, const char* file, int line) {
+    (void) file, (void) line;
+    if (s == 0 || n == 0)
+        Gif_Free(p);
+    else if (s == 1 || n == 1 || s <= ((size_t) -1) / n)
+        return realloc(p, s * n);
+    else
+        return (void*) 0;
+}
+
+#undef Gif_Free
+void Gif_Free(void* p) {
+    free(p);
+}
+#endif
+
 #ifdef __cplusplus
 }
 #endif
