@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /* gifsicle.c - gifsicle's main loop.
-   Copyright (C) 1997-2013 Eddie Kohler, ekohler@cs.ucla.edu
+   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -10,6 +10,7 @@
 
 #include <config.h>
 #include "gifsicle.h"
+#include "kcolor.h"
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,7 +57,7 @@ static int infoing = 0;
 int verbosing = 0;
 
 
-#define CHANGED(next, flag)	(((next) & 1<<(flag)) != 0)
+#define CHANGED(next, flag)	(((next) & (1<<(flag))) != 0)
 #define UNCHECKED_MARK_CH(where, what)			\
   next_##where |= 1<<what;
 #define MARK_CH(where, what)				\
@@ -110,11 +111,13 @@ static int any_output_successful = 0;
 #define CH_RESIZE		10
 #define CH_MEMORY		11
 #define CH_GAMMA		12
+#define CH_RESIZE_METHOD	13
+#define CH_SCALE_COLORS		14
 static const char *output_option_types[] = {
   "loopcount", "logical screen", "optimization", "output file",
   "colormap size", "dither", "colormap", "colormap method",
   "background", "color transformation", "resize", "memory conservation",
-  "gamma"
+  "gamma", "resize method", "resize colors"
 };
 
 
@@ -186,6 +189,8 @@ static const char *output_option_types[] = {
 #define SIZE_INFO_OPT		367
 #define GAMMA_OPT		368
 #define GRAY_OPT		369
+#define RESIZE_METHOD_OPT	370
+#define RESIZE_COLORS_OPT	371
 
 #define LOOP_TYPE		(Clp_ValFirstUser)
 #define DISPOSAL_TYPE		(Clp_ValFirstUser + 1)
@@ -198,6 +203,7 @@ static const char *output_option_types[] = {
 #define COLORMAP_ALG_TYPE	(Clp_ValFirstUser + 8)
 #define SCALE_FACTOR_TYPE	(Clp_ValFirstUser + 9)
 #define OPTIMIZE_TYPE		(Clp_ValFirstUser + 10)
+#define RESIZE_METHOD_TYPE	(Clp_ValFirstUser + 11)
 
 const Clp_Option options[] = {
 
@@ -275,6 +281,8 @@ const Clp_Option options[] = {
   { "resize-fit-height", 0, RESIZE_FIT_HEIGHT_OPT, Clp_ValUnsigned, Clp_Negate },
   { "resize-fi", 0, RESIZE_FIT_OPT, DIMENSIONS_TYPE, Clp_Negate },
   { "resize-f", 0, RESIZE_FIT_OPT, DIMENSIONS_TYPE, Clp_Negate },
+  { "resize-method", 0, RESIZE_METHOD_OPT, RESIZE_METHOD_TYPE, 0 },
+  { "resize-colors", 0, RESIZE_COLORS_OPT, Clp_ValInt, Clp_Negate },
   { "rotate-90", 0, ROTATE_90_OPT, 0, 0 },
   { "rotate-180", 0, ROTATE_180_OPT, 0, 0 },
   { "rotate-270", 0, ROTATE_270_OPT, 0, 0 },
@@ -296,6 +304,8 @@ const Clp_Option options[] = {
   { "same-screen", 0, SAME_LOGICAL_SCREEN_OPT, 0, 0 },
   { "same-transparent", 0, SAME_TRANSPARENT_OPT, 0, 0 },
   { "scale", 0, SCALE_OPT, SCALE_FACTOR_TYPE, Clp_Negate },
+  { "scale-method", 0, RESIZE_METHOD_OPT, RESIZE_METHOD_TYPE, 0 },
+  { "scale-colors", 0, RESIZE_COLORS_OPT, Clp_ValInt, Clp_Negate },
   { "screen", 0, LOGICAL_SCREEN_OPT, DIMENSIONS_TYPE, Clp_Negate },
   { "sinfo", 0, SIZE_INFO_OPT, 0, Clp_Negate },
   { "size-info", 0, SIZE_INFO_OPT, 0, Clp_Negate },
@@ -318,6 +328,8 @@ const Clp_Option options[] = {
 
 };
 
+Clp_Parser* clp;
+
 
 static void combine_output_options(void);
 static void initialize_def_frame(void);
@@ -337,7 +349,7 @@ set_mode(int newmode)
   }
 
   if (mode != INFOING && infoing == 1)
-    fatal_error("'--info' suppresses normal output, can't use with an\n  output mode like '--merge' or '--batch'.\n  (Try '-II', which doesn't suppress normal output.)");
+    fatal_error("%<--info%> suppresses normal output, can%,t use with an\n  output mode like %<--merge%> or %<--batch%>.\n  (Try %<-II%>, which doesn%,t suppress normal output.)");
   if (newmode != BLANK_MODE && newmode != mode)
     fatal_error("too late to change modes");
 }
@@ -351,7 +363,7 @@ set_frame_change(int kind)
 
   set_mode(BLANK_MODE);
   if (mode < DELETING && frames_done) {
-    fatal_error("frame selection and frame changes don't mix");
+    fatal_error("frame selection and frame changes don%,t mix");
     return;
   }
   assert(!nested_mode);
@@ -425,7 +437,7 @@ show_frame(int imagenumber, int usename)
    case BATCHING:
     if (!frames_done)
       clear_frameset(frames, first_input_frame);
-    frame = add_frame(frames, -1, input, gfi);
+    frame = add_frame(frames, input, gfi);
     if (usename)
       frame->explode_by_name = 1;
     break;
@@ -449,35 +461,43 @@ show_frame(int imagenumber, int usename)
 static int gifread_error_count;
 
 static void
-gifread_error(int is_error, const char *message, int which_image, void *thunk)
+gifread_error(Gif_Stream* gfs, Gif_Image* gfi,
+              int is_error, const char *message)
 {
   static int last_is_error = 0;
-  static int last_which_image = 0;
+  static char last_landmark[256];
   static char last_message[256];
   static int different_error_count = 0;
   static int same_error_count = 0;
-  const char *filename = (const char *)thunk;
+  char landmark[256];
+  int which_image = Gif_ImageNumber(gfs, gfi);
+  if (gfs && which_image < 0)
+    which_image = gfs->nimages;
 
   /* ignore warnings if "no_warning" */
   if (no_warnings && is_error == 0)
     return;
 
-  if (gifread_error_count == 0) {
-    last_which_image = -1;
-    last_message[0] = 0;
-    different_error_count = 0;
+  if (message) {
+    const char *filename = gfs && gfs->landmark ? gfs->landmark : "<unknown>";
+    if (gfi && (which_image != 0 || gfs->nimages != 1))
+      snprintf(landmark, sizeof(landmark), "%s:#%d",
+               filename, which_image < 0 ? gfs->nimages : which_image);
+    else
+      snprintf(landmark, sizeof(landmark), "%s", filename);
   }
 
-  gifread_error_count++;
-  if (last_message[0] && different_error_count <= 10
-      && (last_which_image != which_image || message == 0
-	  || strcmp(message, last_message) != 0)) {
-    const char *etype = last_is_error ? "error" : "warning";
-    error(0, "While reading '%s' frame #%d:", filename, last_which_image);
+  if (last_message[0]
+      && different_error_count <= 10
+      && (!message
+          || strcmp(message, last_message) != 0
+          || strcmp(landmark, last_landmark) != 0)) {
+    const char* etype = last_is_error ? "read error: " : "";
+    void (*f)(const char*, const char*, ...) = last_is_error ? lerror : lwarning;
     if (same_error_count == 1)
-      error(0, "  %s: %s", etype, last_message);
+      f(last_landmark, "%s%s", etype, last_message);
     else if (same_error_count > 0)
-      error(0, "  %s: %s (%d times)", etype, last_message, same_error_count);
+      f(last_landmark, "%s%s (%d times)", etype, last_message, same_error_count);
     same_error_count = 0;
     last_message[0] = 0;
   }
@@ -487,14 +507,23 @@ gifread_error(int is_error, const char *message, int which_image, void *thunk)
       different_error_count++;
     same_error_count++;
     strcpy(last_message, message);
-    last_which_image = which_image;
+    strcpy(last_landmark, landmark);
     last_is_error = is_error;
+    if (different_error_count == 11) {
+      error(0, "(plus more errors; is this GIF corrupt?)");
+      different_error_count++;
+    }
   } else
     last_message[0] = 0;
 
-  if (different_error_count == 11 && message) {
-    error(0, "(more errors while reading '%s')", filename);
-    different_error_count++;
+  {
+    unsigned long missing;
+    if (message && sscanf(message, "missing %lu pixels", &missing) == 1
+        && missing > 10000) {
+      gifread_error(gfs, 0, -1, 0);
+      lerror(landmark, "fatal error: too many missing pixels, giving up");
+      exit(1);
+    }
   }
 }
 
@@ -516,7 +545,7 @@ open_giffile(const char *name)
 #ifndef OUTPUT_GIF_TO_TERMINAL
     extern int isatty(int);
     if (isatty(fileno(stdin))) {
-      error(0, "<stdin>: is a terminal");
+      lerror("<stdin>", "is a terminal");
       return NULL;
     }
 #endif
@@ -544,7 +573,7 @@ open_giffile(const char *name)
     stored_files = sf;
     strcpy(sf->name, name);
   } else if (!f)
-    error(0, "%s: %s", name, strerror(errno));
+    lerror(name, "%s", strerror(errno));
 
   return f;
 }
@@ -622,9 +651,9 @@ input_stream(const char *name)
   i = getc(f);
   if (i == EOF) {
     if (!(gif_read_flags & GIF_READ_TRAILING_GARBAGE_OK))
-      error(0, "%s: empty file", name);
+      lerror(name, "empty file");
     else if (nextfile)
-      error(0, "%s: no more images in file", name);
+      lerror(name, "no more images in file");
     close_giffile(f, 1);
     return;
   }
@@ -636,14 +665,13 @@ input_stream(const char *name)
   /* read file */
   gifread_error_count = 0;
   gfs = Gif_FullReadFile(f, gif_read_flags | GIF_READ_COMPRESSED,
-			 gifread_error, (void *)name);
-  gifread_error(-1, 0, -1, (void *)name); /* print out last error message */
+			 name, gifread_error);
 
   if (!gfs || (Gif_ImageCount(gfs) == 0 && gfs->errors > 0)) {
     if (componentno == 1)
-      error(0, "%s: file not in GIF format", name);
+      lerror(name, "file not in GIF format");
     else
-      error(0, "%s: trailing garbage ignored", main_name);
+      lerror(name, "trailing garbage ignored");
     Gif_DeleteStream(gfs);
     if (verbosing)
       verbose_close('>');
@@ -654,7 +682,7 @@ input_stream(const char *name)
   /* special processing for components after the first */
   if (componentno > 1) {
     if (mode == BATCHING || mode == INSERTING)
-      fatal_error("%s: --multifile is useful only in merge mode");
+      fatal_error("%s: %<--multifile%> is useful only in merge mode", name);
     input_done();
   }
 
@@ -709,21 +737,21 @@ input_stream(const char *name)
 
   old_def_frame = def_frame;
   first_input_frame = frames->count;
-  if (gfs->nimages > 1)
-    def_frame.position_is_offset = 1;
+  def_frame.position_is_offset = 1;
   for (i = 0; i < gfs->nimages; i++)
-    add_frame(frames, -1, gfs, gfs->images[i]);
+    add_frame(frames, gfs, gfs->images[i]);
   def_frame = old_def_frame;
 
   if (unoptimizing)
     if (!Gif_FullUnoptimize(gfs, GIF_UNOPTIMIZE_SIMPLEST_DISPOSAL)) {
       static int context = 0;
-      warning(1, "GIF too complex to unoptimize", name);
       if (!context) {
-	warncontext(1, "(The reason was local color tables or complex transparency.");
-	warncontext(1, "Try running the GIF through 'gifsicle --colors=255' first.)");
-      }
-      context = 1;
+        lwarning(name, "GIF too complex to unoptimize\n"
+                 "  (The reason was local color tables or complex transparency.\n"
+                 "  Try running the GIF through %<gifsicle --colors=255%> first.)");
+        context = 1;
+      } else
+        lwarning(name, "GIF too complex to unoptimize");
     }
 
   apply_color_transforms(input_transforms, gfs);
@@ -800,50 +828,47 @@ do_colormap_change(Gif_Stream *gfs)
                     &active_output_data);
 
   if (active_output_data.colormap_size > 0) {
-    int nhist;
-    Gif_Color *hist;
-    Gif_Colormap* (*adapt_func)(Gif_Color*, int, Gt_OutputData*);
+    kchist kch;
+    Gif_Colormap* (*adapt_func)(kchist*, Gt_OutputData*);
     Gif_Colormap *new_cm;
 
     /* set up the histogram */
     {
+      uint32_t ntransp;
       int i, any_locals = 0;
       for (i = 0; i < gfs->nimages; i++)
 	if (gfs->images[i]->local)
 	  any_locals = 1;
-      hist = histogram(gfs, &nhist);
-      if (nhist <= active_output_data.colormap_size
+      kchist_make(&kch, gfs, &ntransp);
+      if (kch.n <= active_output_data.colormap_size
           && !any_locals
           && !active_output_data.colormap_fixed) {
-	warning(1, "trivial adaptive palette (only %d colors in source)", nhist);
+	warning(1, "trivial adaptive palette (only %d colors in source)", kch.n);
+        kchist_cleanup(&kch);
 	return;
       }
+      active_output_data.colormap_needs_transparency = ntransp > 0;
     }
 
     switch (active_output_data.colormap_algorithm) {
-
      case COLORMAP_DIVERSITY:
       adapt_func = &colormap_flat_diversity;
       break;
-
      case COLORMAP_BLEND_DIVERSITY:
       adapt_func = &colormap_blend_diversity;
       break;
-
      case COLORMAP_MEDIAN_CUT:
       adapt_func = &colormap_median_cut;
       break;
-
      default:
       fatal_error("can't happen");
-
     }
 
-    new_cm = (*adapt_func)(hist, nhist, &active_output_data);
+    new_cm = (*adapt_func)(&kch, &active_output_data);
     colormap_stream(gfs, new_cm, &active_output_data);
 
-    Gif_DeleteArray(hist);
     Gif_DeleteColormap(new_cm);
+    kchist_cleanup(&kch);
   }
 }
 
@@ -863,7 +888,7 @@ write_stream(const char *output_name, Gif_Stream *gfs)
 #ifndef OUTPUT_GIF_TO_TERMINAL
     extern int isatty(int);
     if (isatty(fileno(stdout))) {
-      error(0, "<stdout>: is a terminal");
+      lerror("<stdout>", "is a terminal");
       return;
     }
 #endif
@@ -883,7 +908,7 @@ write_stream(const char *output_name, Gif_Stream *gfs)
     fclose(f);
     any_output_successful = 1;
   } else
-    error(0, "%s: %s", output_name, strerror(errno));
+    lerror(output_name, "%s", strerror(errno));
 }
 
 static void
@@ -913,15 +938,19 @@ merge_and_write_frames(const char *outfile, int f1, int f2)
 			     compress_immediately, &huge_stream);
 
   if (out) {
-    if (active_output_data.scaling == GT_SCALING_RESIZE)
-      resize_stream(out, active_output_data.resize_width,
-		    active_output_data.resize_height, 0);
-    else if (active_output_data.scaling == GT_SCALING_SCALE)
-      resize_stream(out, active_output_data.scale_x * out->screen_width,
-		    active_output_data.scale_y * out->screen_height, 0);
-    else if (active_output_data.scaling == GT_SCALING_RESIZE_FIT)
-      resize_stream(out, active_output_data.resize_width,
-		    active_output_data.resize_height, 1);
+    double w, h;
+    if (active_output_data.scaling == GT_SCALING_SCALE) {
+      w = active_output_data.scale_x * out->screen_width;
+      h = active_output_data.scale_y * out->screen_height;
+    } else {
+      w = active_output_data.resize_width;
+      h = active_output_data.resize_height;
+    }
+    if (active_output_data.scaling != GT_SCALING_NONE)
+      resize_stream(out, w, h,
+                    active_output_data.scaling == GT_SCALING_RESIZE_FIT,
+                    active_output_data.scale_method,
+                    active_output_data.scale_colors);
     if (colormap_change)
       do_colormap_change(out);
     if (output_transforms)
@@ -932,7 +961,8 @@ merge_and_write_frames(const char *outfile, int f1, int f2)
     Gif_DeleteStream(out);
   }
 
-  if (verbosing) verbose_close(']');
+  if (verbosing)
+    verbose_close(']');
   active_output_data.active_output_name = 0;
 }
 
@@ -951,7 +981,7 @@ output_information(const char *outfile)
   else {
     f = fopen(outfile, "w");
     if (!f) {
-      error(0, "%s: %s", outfile, strerror(errno));
+      lerror(outfile, "%s", strerror(errno));
       return;
     }
   }
@@ -1081,14 +1111,14 @@ handle_extension(Clp_Parser *clp, int is_app)
 
   UNCHECKED_MARK_CH(frame, CH_EXTENSION);
   if (is_app)
-    gfex = Gif_NewExtension(255, extension_type);
+    gfex = Gif_NewExtension(255, extension_type, 11);
   else if (!isdigit(extension_type[0]) && extension_type[1] == 0)
-    gfex = Gif_NewExtension(extension_type[0], 0);
+    gfex = Gif_NewExtension(extension_type[0], 0, 0);
   else {
     long l = strtol(extension_type, (char **)&extension_type, 0);
     if (*extension_type != 0 || l < 0 || l >= 256)
       fatal_error("bad extension type: must be a number between 0 and 255");
-    gfex = Gif_NewExtension(l, 0);
+    gfex = Gif_NewExtension(l, 0, 0);
   }
 
   gfex->data = (uint8_t *)extension_body;
@@ -1156,6 +1186,8 @@ initialize_def_frame(void)
 
   def_output_data.optimizing = 0;
   def_output_data.scaling = GT_SCALING_NONE;
+  def_output_data.scale_method = SCALE_METHOD_MIX;
+  def_output_data.scale_colors = 0;
 
   def_output_data.conserve_memory = 0;
 
@@ -1213,6 +1245,16 @@ combine_output_options(void)
     active_output_data.scale_y = def_output_data.scale_y;
   }
 
+  if (CHANGED(recent, CH_RESIZE_METHOD)) {
+    MARK_CH(output, CH_RESIZE_METHOD);
+    active_output_data.scale_method = def_output_data.scale_method;
+  }
+
+  if (CHANGED(recent, CH_SCALE_COLORS)) {
+    MARK_CH(output, CH_SCALE_COLORS);
+    active_output_data.scale_colors = def_output_data.scale_colors;
+  }
+
   COMBINE_ONE_OUTPUT_OPTION(CH_MEMORY, conserve_memory);
 
   def_output_data.colormap_fixed = 0;
@@ -1223,16 +1265,16 @@ combine_output_options(void)
 }
 
 static void
-redundant_option_warning(const char *option_type)
+redundant_option_warning(const char* opttype)
 {
   static int context = 0;
-  warning(0, "redundant %s option", option_type);
   if (!context) {
-    warncontext(0, "(The %s option was overridden by another %s option",
-		option_type, option_type);
-    warncontext(0, "before it had any effect.)");
-  }
-  context = 1;
+    warning(0, "redundant %s option\n"
+            "  (The %s option was overridden by another %s option\n"
+            "  before it had any effect.)", opttype, opttype, opttype);
+    context = 1;
+  } else
+    warning(0, "redundant %s option", opttype);
 }
 
 static void
@@ -1243,10 +1285,12 @@ print_useless_options(const char *type_name, int value, const char *names[])
   if (!value) return;
   for (i = 0; i < 32; i++)
     if (CHANGED(value, i)) {
-      warning(0, "useless %s-related %s option", names[i], type_name);
-      if (!explanation_printed)
-	warncontext(0, "(It didn't affect any %s.)", type_name);
-      explanation_printed = 1;
+      if (!explanation_printed) {
+        warning(0, "useless %s-related %s option\n"
+                "  (It didn%,t affect any %s.)", names[i], type_name, type_name);
+        explanation_printed = 1;
+      } else
+        warning(0, "useless %s-related %s option", names[i], type_name);
     }
 }
 
@@ -1271,8 +1315,6 @@ copy_crop(Gt_Crop *oc)
 int
 main(int argc, char *argv[])
 {
-  Clp_Parser* clp;
-
   /* Check SIZEOF constants (useful for Windows). If these assertions fail,
      you've used the wrong Makefile. You should've used Makefile.w32 for
      32-bit Windows and Makefile.w64 for 64-bit Windows. */
@@ -1307,6 +1349,20 @@ main(int argc, char *argv[])
      "drop-empty", GT_OPT_KEEPEMPTY,
      "no-drop-empty", GT_OPT_KEEPEMPTY + 1,
      (const char*) 0);
+  Clp_AddStringListType
+    (clp, RESIZE_METHOD_TYPE, 0,
+     "point", SCALE_METHOD_POINT,
+     "sample", SCALE_METHOD_POINT,
+     "mix", SCALE_METHOD_MIX,
+     "box", SCALE_METHOD_BOX,
+     "catrom", SCALE_METHOD_CATROM,
+     "lanczos", SCALE_METHOD_LANCZOS3,
+     "lanczos2", SCALE_METHOD_LANCZOS2,
+     "lanczos3", SCALE_METHOD_LANCZOS3,
+     "mitchell", SCALE_METHOD_MITCHELL,
+     "fast", SCALE_METHOD_POINT,
+     "good", SCALE_METHOD_MIX,
+     (const char*) 0);
   Clp_AddType(clp, DIMENSIONS_TYPE, 0, parse_dimensions, 0);
   Clp_AddType(clp, POSITION_TYPE, 0, parse_position, 0);
   Clp_AddType(clp, SCALE_FACTOR_TYPE, 0, parse_scale_factor, 0);
@@ -1322,10 +1378,7 @@ main(int argc, char *argv[])
   frames = new_frameset(16);
   initialize_def_frame();
   Gif_InitCompressInfo(&gif_write_info);
-
-#ifdef DMALLOC
-  dmalloc_verbose("fudge");
-#endif
+  Gif_SetErrorHandler(gifread_error);
 
   /* Yep, I'm an idiot.
      GIF dimensions are unsigned 16-bit integers. I assume that these
@@ -1682,20 +1735,20 @@ main(int argc, char *argv[])
      }
 
      case CHANGE_COLOR_OPT: {
-       next_input |= CH_CHANGE_COLOR;
+       next_input |= 1 << CH_CHANGE_COLOR;
        if (clp->negated)
 	 input_transforms = delete_color_transforms
 	   (input_transforms, &color_change_transformer);
        else if (parsed_color2.haspixel)
-	 error(0, "COLOR2 must be in RGB format in '--change-color COLOR1 COLOR2'");
+	 error(0, "COLOR2 must be in RGB format in %<--change-color COLOR1 COLOR2%>");
        else
 	 input_transforms = append_color_change
 	   (input_transforms, parsed_color, parsed_color2);
        break;
      }
 
-     case COLOR_TRANSFORM_OPT:
-      next_output |= CH_COLOR_TRANSFORM;
+    case COLOR_TRANSFORM_OPT:
+      next_output |= 1 << CH_COLOR_TRANSFORM;
       if (clp->negated)
 	output_transforms = delete_color_transforms
 	  (output_transforms, &pipe_color_transformer);
@@ -1789,7 +1842,7 @@ main(int argc, char *argv[])
       if (clp->negated)
 	def_output_data.scaling = GT_SCALING_NONE;
       else if (dimensions_x <= 0 && dimensions_y <= 0) {
-	error(0, "one of W and H must be positive in '%s WxH'", Clp_CurOptionName(clp));
+	error(0, "one of W and H must be positive in %<%s WxH%>", Clp_CurOptionName(clp));
 	def_output_data.scaling = GT_SCALING_NONE;
       } else {
 	def_output_data.scaling = (opt == RESIZE_FIT_OPT ? GT_SCALING_RESIZE_FIT : GT_SCALING_RESIZE);
@@ -1828,12 +1881,26 @@ main(int argc, char *argv[])
 	error(0, "%s X and Y factors must be positive", Clp_CurOptionName(clp));
 	def_output_data.scaling = GT_SCALING_NONE;
       } else {
-	def_output_data.scaling = 2; /* use scale factor */
+	def_output_data.scaling = GT_SCALING_SCALE;
 	def_output_data.scale_x = parsed_scale_factor_x;
 	def_output_data.scale_y = parsed_scale_factor_y;
       }
       break;
 
+    case RESIZE_METHOD_OPT:
+      MARK_CH(output, CH_RESIZE_METHOD);
+      def_output_data.scale_method = clp->val.i;
+      break;
+
+    case RESIZE_COLORS_OPT:
+      MARK_CH(output, CH_SCALE_COLORS);
+      def_output_data.scale_colors = clp->negated ? 0 : clp->val.i;
+      if (def_output_data.scale_colors > 256) {
+        error(0, "%s can be at most 256", Clp_CurOptionName(clp));
+        def_output_data.scale_colors = 256;
+      }
+      break;
+
       /* RANDOM OPTIONS */
 
      case NO_WARNINGS_OPT:
@@ -1873,7 +1940,7 @@ main(int argc, char *argv[])
 #else
       printf("LCDF Gifsicle %s\n", VERSION);
 #endif
-      printf("Copyright (C) 1997-2013 Eddie Kohler\n\
+      printf("Copyright (C) 1997-2014 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -1935,9 +2002,6 @@ particular purpose.\n");
   if (any_output_successful)
     print_useless_options("output", active_next_output, output_option_types);
   blank_frameset(frames, 0, 0, 1);
-#ifdef DMALLOC
-  dmalloc_report();
-#endif
   Clp_DeleteParser(clp);
   return (error_count ? EXIT_ERR : EXIT_OK);
 }
