@@ -1,5 +1,5 @@
 /* gifsicle.h - Function declarations for gifsicle.
-   Copyright (C) 1997-2011 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -7,6 +7,8 @@
    as this notice is kept intact and this source code is made available. There
    is no warranty, express or implied. */
 
+#ifndef GIFSICLE_H
+#define GIFSICLE_H
 #include <lcdfgif/gif.h>
 #include <lcdf/clp.h>
 #ifdef __GNUC__
@@ -107,6 +109,7 @@ typedef struct {
   int colormap_size;
   Gif_Colormap *colormap_fixed;
   int colormap_algorithm;
+  int colormap_needs_transparency;
   int dither_type;
   const uint8_t* dither_data;
   const char* dither_name;
@@ -120,18 +123,29 @@ typedef struct {
   int resize_height;
   double scale_x;
   double scale_y;
+  int scale_method;
+  int scale_colors;
 
   int conserve_memory;
 
 } Gt_OutputData;
 
 extern Gt_OutputData active_output_data;
+extern Clp_Parser* clp;
 
 #define GT_SCALING_NONE		0
 #define GT_SCALING_RESIZE	1
 #define GT_SCALING_SCALE	2
 #define GT_SCALING_RESIZE_FIT	3
 
+#define SCALE_METHOD_POINT	0
+#define SCALE_METHOD_BOX	1
+#define SCALE_METHOD_MIX	2
+#define SCALE_METHOD_CATROM	3
+#define SCALE_METHOD_LANCZOS2	4
+#define SCALE_METHOD_LANCZOS3	5
+#define SCALE_METHOD_MITCHELL	6
+
 #define GT_OPT_MASK		0xFFFF
 #define GT_OPT_KEEPEMPTY	0x10000
 
@@ -156,10 +170,11 @@ extern int error_count;
 extern int no_warnings;
 extern Gif_CompressInfo gif_write_info;
 
-void fatal_error(const char *message, ...) NORETURN;
-void warning(int need_file, const char *message, ...);
-void warncontext(int need_file, const char *message, ...);
-void error(int need_file, const char *message, ...);
+void fatal_error(const char* format, ...) NORETURN;
+void warning(int need_file, const char* format, ...);
+void lwarning(const char* landmark, const char* format, ...);
+void error(int need_file, const char* format, ...);
+void lerror(const char* landmark, const char* format, ...);
 void clp_error_handler(Clp_Parser *clp, const char *clp_message);
 void usage(void);
 void short_usage(void);
@@ -199,8 +214,8 @@ int	merge_colormap_if_possible(Gif_Colormap *, Gif_Colormap *);
 extern int warn_local_colormaps;
 void	merge_stream(Gif_Stream *dest, Gif_Stream *src, int no_comments);
 void	merge_comments(Gif_Comment *destc, Gif_Comment *srcc);
-Gif_Image *merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci,
-		       int same_compressed_ok);
+Gif_Image* merge_image(Gif_Stream* dest, Gif_Stream* src, Gif_Image* srci,
+		       Gt_Frame* srcfr, int same_compressed_ok);
 
 void	optimize_fragments(Gif_Stream *, int optimizeness, int huge_stream);
 
@@ -227,13 +242,12 @@ int	crop_image(Gif_Image *, Gt_Crop *, int preserve_total_crop);
 
 void	flip_image(Gif_Image *, int scr_width, int scr_height, int is_vert);
 void	rotate_image(Gif_Image *, int scr_width, int scr_height, int rotation);
-void	resize_stream(Gif_Stream *, double new_width, double new_height,
-                      int fit);
+void	resize_stream(Gif_Stream* gfs, double new_width, double new_height,
+                      int fit, int method, int scale_colors);
 
 /*****
  * quantization
  **/
-Gif_Color *histogram(Gif_Stream *, int *);
 #define KC_GAMMA_SRGB			0
 #define KC_GAMMA_NUMERIC		1
 void    kc_set_gamma(int type, double gamma);
@@ -241,11 +255,6 @@ void    kc_set_gamma(int type, double gamma);
 #define COLORMAP_DIVERSITY		0
 #define COLORMAP_BLEND_DIVERSITY	1
 #define COLORMAP_MEDIAN_CUT		2
-Gif_Colormap* colormap_blend_diversity(Gif_Color*, int, Gt_OutputData*);
-Gif_Colormap* colormap_flat_diversity(Gif_Color*, int, Gt_OutputData*);
-Gif_Colormap* colormap_median_cut(Gif_Color*, int, Gt_OutputData*);
-
-typedef struct kd3_tree kd3_tree;
 
 enum {
     dither_none = 0, dither_default, dither_floyd_steinberg,
@@ -291,7 +300,7 @@ extern Gt_Frame def_frame;
 #define		FRAME(fs, i)	((fs)->f[i])
 
 Gt_Frameset *	new_frameset(int initial_cap);
-Gt_Frame *	add_frame(Gt_Frameset *, int num, Gif_Stream *, Gif_Image *);
+Gt_Frame*	add_frame(Gt_Frameset*, Gif_Stream*, Gif_Image*);
 void		clear_def_frame_once_options(void);
 
 Gif_Stream *	merge_frame_interval(Gt_Frameset *, int f1, int f2,
@@ -312,3 +321,5 @@ void		blank_frameset(Gt_Frameset *, int from, int to, int delete_ob);
 
 extern int mode;
 extern int nested_mode;
+
+#endif
