@@ -2,7 +2,7 @@
 /* ungifwrt.c - Functions to write unGIFs -- GIFs with run-length compression,
    not LZW compression. Idea due to Hutchinson Avenue Software Corporation
    <http://www.hasc.com>.
-   Copyright (C) 1997-2013 Eddie Kohler, kohler@cs.ucla.edu
+   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of the LCDF GIF library.
 
    The LCDF GIF library is free software. It is distributed under the GNU
@@ -394,7 +394,7 @@ calculate_min_code_bits(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
   if ((grr->gcinfo.flags & GIF_WRITE_CAREFUL_MIN_CODE_SIZE)
       && gfi->compressed && gfi->compressed[0] != min_code_bits) {
     /* if compressed image disagrees with careful min_code_bits, recompress */
-    if (Gif_UncompressImage(gfi))
+    if (Gif_UncompressImage(gfs, gfi))
       Gif_FullCompressImage(gfs, gfi, &grr->gcinfo);
   }
 
@@ -473,7 +473,7 @@ Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi,
   }
   gfi->compressed = grr.v;
   gfi->compressed_len = grr.pos;
-  gfi->free_compressed = Gif_DeleteArrayFunc;
+  gfi->free_compressed = Gif_Free;
   Gif_DeleteArray(gfc.rle_next);
   return grr.v != 0;
 }
